#!/usr/bin/python

################################################################
#                                                              #
# Blue Team Training Toolkit (BT3)                             #
# written by Juan J. Guelfo @ Encripto AS                      #
# post@encripto.no                                             #
#                                                              #
# Copyright 2013-2018 Encripto AS. All rights reserved.        #
#                                                              #
# BT3 is licensed under the FreeBSD license.                   #
# http://www.freebsd.org/copyright/freebsd-license.html        #
#                                                              #
################################################################


import os, zlib, zipfile
import libs.bt3api, libs.bt3in, libs.bt3out


# Common routine used by all modules for "download" command autocompletion
def autocomplete_download(api_credentials, text, line, material_type):
    autocomplete_list = []
    if len(api_credentials) == 2 and line.startswith("download"):
        try:
            json_results = libs.bt3api.get_material_list(api_credentials[0], api_credentials[1], material_type)
            if json_results and libs.bt3api.validate_json(json_results):
                parsed_results = libs.bt3api.parse_json(json_results)
                if parsed_results[-1]["Result"]:
                    del parsed_results[-1]
                    for result in parsed_results:
                        autocomplete_list.append(result["Name"])

        except Exception as e:
            print("")
            libs.bt3out.print_error("%s\n" % e)

    return [i for i in autocomplete_list if i.startswith(text)]


# Common routine used by all modules for "info" command autocompletion
def autocomplete_info(api_credentials, text, line, material_type, material_folder):
    autocomplete_list = []
    if line.startswith("info"):
        try:
            files = [f for f in os.listdir(material_folder) if os.path.isfile(os.path.join(material_folder, f))]
            files.sort()
            for material_file in files:
                if material_type.upper() in ["PROFILE"] and material_file.endswith(".py") and material_file not in ["__init__.py"]:
                    autocomplete_list.append(material_file)

                elif material_type.upper() in ["PCAP"] and (material_file.endswith(".pcap") or material_file.endswith(".cap")):
                    autocomplete_list.append(material_file)

                elif material_type.upper() in ["MOCKFILE"] and material_file.endswith(".mock"):
                    autocomplete_list.append(material_file)

        except Exception as e:
            print("")
            libs.bt3out.print_error("%s\n" % e)

    if len(api_credentials) == 2 and line.startswith("info"):
        try:
            json_results = libs.bt3api.get_material_list(api_credentials[0], api_credentials[1], material_type)
            if json_results and libs.bt3api.validate_json(json_results):
                parsed_results = libs.bt3api.parse_json(json_results)
                if parsed_results[-1]["Result"]:
                    del parsed_results[-1]
                    for result in parsed_results:
                        autocomplete_list.append(result["Name"])

        except Exception as e:
            print("")
            libs.bt3out.print_error("%s\n" % e)

    return [i for i in autocomplete_list if i.startswith(text)]


# Common routine used by all modules for "show" command autocompletion
def autocomplete_show(text, line, material_type):
    autocomplete_list = ["options", "interfaces", "downloads"]

    if material_type.upper() in ["PROFILE"]:
        autocomplete_list.append("profiles")

    elif material_type.upper() in ["PCAP"]:
        autocomplete_list.append("pcaps")

    elif material_type.upper() in ["MOCKFILE"]:
        autocomplete_list.append("mockfiles")

    if line.startswith("show profiles") or line.startswith("show pcaps") or line.startswith("show mockfiles"):
        autocomplete_list.append("cloud")
        autocomplete_list.append("disk")
        autocomplete_list.append("free")
        autocomplete_list.append("premium")

    return [i for i in autocomplete_list if i.startswith(text)]


# Common routine used by all modules for "download" command
def cmd_download(args, api_credentials, material_type, material_folder):
    if len(api_credentials) == 2:
        args = args.split(" ")
        if len(args) == 1 and args[0] not in [""]:

            # Display download cost
            download_cost = None
            fetched_cost = False
            json_results = libs.bt3api.get_material_cost(api_credentials[0], api_credentials[1], material_type, args[0])
            if json_results and libs.bt3api.validate_json(json_results):
                parsed_results = libs.bt3api.parse_json(json_results)
                if parsed_results[-1]["Result"]:
                    del parsed_results[-1]
                    for result in parsed_results:
                        download_cost = result["Price"]
                        fetched_cost = True
                else:
                    print("")
                    libs.bt3out.print_error("%s" % libs.bt3out.MATERIAL_DOWNLOAD_ERROR)
                    libs.bt3out.print_error("%s\n" % libs.bt3out.CONTACT_SUPPORT)

            else:
                print("")
                libs.bt3out.print_error("%s" % libs.bt3out.API_INCOMMUNICATION)
                libs.bt3out.print_error("%s\n" % libs.bt3out.CONTACT_SUPPORT)

            # Ask for download confirmation
            if fetched_cost and float(download_cost) >= 0 and libs.bt3in.get_confirmation("This download will deduct %s credit(s) from your content credit balance." % download_cost, "WARN"):
                try:
                    libs.bt3out.print_info("Downloading...")
                    json_results = libs.bt3api.download_material(api_credentials[0], api_credentials[1], material_type, args[0], material_folder)
                    if json_results and libs.bt3api.validate_json(json_results):
                        parsed_results = libs.bt3api.parse_json(json_results)
                        if not parsed_results[-1]["Result"]:
                            libs.bt3out.print_error("%s\n" % parsed_results[-1]["Msg"])
                            libs.bt3out.log_download(material_type, args[0], "ERROR")

                    else:
                        zip_file_path = "%s/%s.zip" % (material_folder, os.path.basename(args[0]))
                        if libs.bt3in.check_file_exists(zip_file_path) and zipfile.is_zipfile(zip_file_path):
                            zip_file = zipfile.ZipFile(zip_file_path, "r")
                            zip_file.extractall(material_folder)
                            zip_file.close()
                            os.remove(zip_file_path)
                            libs.bt3out.print_ok("%s\n" % libs.bt3out.MATERIAL_INSTALLED)
                            libs.bt3out.log_download(material_type, args[0], "SUCCESS")

                        else:
                            libs.bt3out.print_error("%s" % libs.bt3out.MATERIAL_DAMAGED)
                            libs.bt3out.log_download(material_type, args[0], "ERROR")

                    # Display credit balance
                    json_results = libs.bt3api.get_subscription(api_credentials[0], api_credentials[1])
                    if json_results and libs.bt3api.validate_json(json_results):
                        parsed_results = libs.bt3api.parse_json(json_results)
                        if parsed_results[-1]["Result"]:
                            del parsed_results[-1]
                            for subscription in parsed_results:
                                libs.bt3out.print_info("Your current balance is %s credit(s).\n" % subscription["CreditBalance"])

                        else:
                            print("")
                            libs.bt3out.print_error("%s\n" % parsed_results[-1]["Msg"])

                    else:
                        print("")
                        libs.bt3out.print_error("Could not retrieve your current balance.\n")

                except Exception as e:
                    print("")
                    libs.bt3out.print_error("%s\n" % e)

        else:
            if material_type.upper() in ["PROFILE"]:
                cmds = [["download <profile>", "Download a given profile from the Blue Team Training Toolkit cloud."]]
                libs.bt3out.print_help(cmds, True, True)

            elif material_type.upper() in ["PCAP"]:
                cmds = [["download <pcap>", "Download a given PCAP file from the Blue Team Training Toolkit cloud."]]
                libs.bt3out.print_help(cmds, True, True)

            elif material_type.upper() in ["MOCKFILE"]:
                cmds = [["download <mockfile>", "Download a given mock file from the Blue Team Training Toolkit cloud."]]
                libs.bt3out.print_help(cmds, True, True)

    else:
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.API_AUTHENTICATION)


def cmd_show_downloads(api_credentials, material_type):
    if len(api_credentials) == 2:
        try:
            result = []
            api_error = False
            json_results = libs.bt3api.get_material_download_list(api_credentials[0], api_credentials[1], material_type)
            if json_results and libs.bt3api.validate_json(json_results):
                parsed_results = libs.bt3api.parse_json(json_results)
                if parsed_results[-1]["Result"]:
                    del parsed_results[-1]
                    for download in parsed_results:
                        if material_type.upper() in ["PROFILE"]:
                            result.append([ download["Name"], "Malware indicator profile", download["Date"] ])
                        
                        elif material_type.upper() in ["PCAP"]:
                            result.append([ download["Name"], "PCAP file", download["Date"] ])
                        
                        elif material_type.upper() in ["MOCKFILE"]:
                            result.append([ download["Name"], "Mock file", download["Date"] ])

            else:
                print("")
                api_error = True
                libs.bt3out.print_error("%s" % libs.bt3out.API_INCOMMUNICATION)
                libs.bt3out.print_error("%s" % libs.bt3out.CONTACT_SUPPORT)

            if len(result) > 0:
                libs.bt3out.print_download_list(result)
                print("")
                libs.bt3out.print_info("Downloads: %s\n" % len(result))

            else:
                print("")
                if material_type.upper() in ["PROFILE"] and not api_error:
                    libs.bt3out.print_info("%s\n" % libs.bt3out.NO_PROFILE_DOWNLOADS)
                    
                elif material_type.upper() in ["PCAP"] and not api_error:
                    libs.bt3out.print_info("%s\n" % libs.bt3out.NO_PCAP_DOWNLOADS)

                elif material_type.upper() in ["MOCKFILE"] and not api_error:
                    libs.bt3out.print_info("%s\n" % libs.bt3out.NO_MOCKFILE_DOWNLOADS)

        except Exception as e:
            print("")
            libs.bt3out.print_error("%s\n" % e)

    else:
        print("")
        libs.bt3out.print_error("%s\n" % libs.bt3out.API_AUTHENTICATION)
